﻿
//====================================================
// Описание работы классов и методов исходника на:
// https://www.interestprograms.ru
// Исходные коды программ и игр
// Автор исходных кодов Клочков Павел
//====================================================
class MyAjax {
    counter: boolean = true;
    constructor() {
        this.initUIJson();
    }

    /**
     * Подготовка интерфейса для отправки сообщений формата JSON
     */
    initUIJson() {
        // Регистрируем обработчик события отправки
        // данных в формате JSON.
        const btnJson = document.getElementById("btnJson");
        btnJson.onclick = () => {
            this.counter == true ? this.jsonFetch() : this.jsonAjax();
            this.counter = !this.counter;
        };
    }

    /**
     * Отправка сообщение формата JSON
     * используя Fetch API
     */
    jsonFetch() {
        const url = '/application-json';

        const options = {
            method: 'POST',
            headers: {
                // Возврат токена на сервер для проверки на 
                // фальшивые запросы от стороннего сайта(XSRF или CSRF).
                'RequestVerificationToken': this.getToken(),
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(this.sendJson("Fetch"))
        };

        fetch(url, options)
            //.then(response => response.json())
            .then(response => response.text())
            .then(data => {
                // Обрабатываем ответ сервера
                this.receiveJson(data);
                //alert(data.api);
            })
            .catch(error => {
                // Обрабатываем ошибку
                console.error("error: " + error);
            });
    }


    /**
    * Отправка сообщение формата JSON
    * используя AJAX API
    */
    jsonAjax() {
        // Для доступа к объекту текущего класса в теле другого объекта.
        const myajax_this = this;

        const url = '/application-json';

        // Создаем объект AJAX XMLHttpRequest
        var xhr = new XMLHttpRequest();
        xhr.open('POST', url, true);
        xhr.setRequestHeader('Content-Type', 'application/json');
        // Возврат токена на сервер для проверки на
        // подделку запросов от стороннего сайта(XSRF или CSRF).
        xhr.setRequestHeader('RequestVerificationToken', this.getToken());
        xhr.onreadystatechange = function () {
            if (xhr.readyState == 4 && xhr.status == 200) {
                // Получаем ответ сервера
                myajax_this.receiveJson(xhr.responseText);
            }
        };
        // Преобразуем данные в строку JSON перед отправкой
        var jsonStr = JSON.stringify(this.sendJson("AJAX"));
        xhr.send(jsonStr);
    }

    /**
     * Получение токена безопасности со страницы
     * @returns
     */
    getToken() {
        const token = document.getElementsByName("__RequestVerificationToken") as unknown as HTMLInputElement;
        return token[0].value;
    }

    /**
     * JSON отправка на сервер
     * @param api
     * @returns
     */
    sendJson(api: string): any {
        return {
            "api": api,
            "inputText": (document.getElementById("inputText") as HTMLInputElement)?.value,
            "inputNumber": (document.getElementById("inputNumber") as HTMLInputElement)?.value,
            "inputCheck": (document.getElementById("inputCheck") as HTMLInputElement)?.checked,
        };
    }

    /**
     * Вывод JSON-сообщений от сервера
     * 
     * @param jsonStr
     */
    receiveJson(jsonStr: string) {
        const json_object = JSON.parse(jsonStr);

        const resultApi = document.getElementById("resultApi");
        resultApi.innerText = json_object.api;
        const resultText = document.getElementById("resultText");
        resultText.innerText = json_object.inputText;
        const resultNumber = document.getElementById("resultNumber");
        resultNumber.innerText = json_object.inputNumber;
        const resultCheck = document.getElementById("resultCheck");
        resultCheck.innerText = json_object.inputCheck;
    }
}
const myAjax = new MyAjax();